﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace ControlWrapsPoC.Wraps
{
    [Designer(typeof(FixedSizeDesigner))]
    public sealed partial class TimePicker : UserControl
    {
        private TimeSpan _value;

        public TimePicker()
        {
            InitializeComponent();
            UnderlyingTimePicker.MouseWheel += UnderlyingTimePicker_MouseWheel;
        }

        private static void UnderlyingTimePicker_MouseWheel(object sender, MouseEventArgs e)
        {
            SendKeys.Send(e.Delta > 0 ? "{UP}" : "{DOWN}");
        }

        public TimeSpan Value
        {
            get
            {
                var truncate = new TimeSpan(_value.Hours, _value.Minutes, 0);
                return truncate;
            }
            set
            {
                var truncate = new TimeSpan(value.Hours, value.Minutes, 0);
                var noChg = truncate == _value;
                if (noChg) return;
                _value = truncate;
                UnderlyingTimePicker.Value = DateTime.Now.Date.Add(value);
                OnValueChanged();
            }
        }

        public event EventHandler ValueChanged;

        private void OnValueChanged()
        {
            ValueChanged?.Invoke(this, EventArgs.Empty);
        }

        private void UnderlyingTimePicker_ValueChanged(object sender, EventArgs e)
        {
            var value = UnderlyingTimePicker.Value.TimeOfDay;
            var truncate = new TimeSpan(value.Hours, value.Minutes, 0);
            if (_value == truncate) return;
            _value = truncate;
            OnValueChanged();
        }
    }

    public class FixedSizeDesigner : ControlDesigner
    {
        public override SelectionRules SelectionRules => SelectionRules.Moveable;
    }
}