﻿using System;
using System.Drawing;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ControlWrapsPoC.Wraps
{
    public static class ToolTipWrap
    {
        // 按调用创建，指定消失时间，消失后释放组件
        // 是否为气泡，图标，标题，内容，控件，[偏移坐标]

        private static void ShowInternal(int durationMs, bool isBalloon, ToolTipIcon icon, string title, string content, Control showOn, Point? position = null)
        {
            var tip = new ToolTip
            {
                UseAnimation = true,
                UseFading = true,
                ShowAlways = true,
                ToolTipIcon = icon,
                ToolTipTitle = title,
                IsBalloon = isBalloon
            };

            //原生控件固有缺陷？ 需要两次才能正确显示气泡，否则会颠倒

            if (position.HasValue)
            {
                tip.Show(content, showOn, position.Value);
                if (isBalloon) tip.Show(content, showOn, position.Value);
            }
            else
            {
                tip.Show(content, showOn);
                if (isBalloon) tip.Show(content, showOn);
            }

            Task.Factory.StartNew(() =>
            {
                Thread.Sleep(durationMs);
                showOn.Invoke(new Action(() => tip.Hide(showOn)));
                Thread.Sleep(500);
                showOn.Invoke(new Action(() => tip.Dispose()));
            });
        }

        public static void Show(int durationMs, bool isBalloon, ToolTipIcon icon, string title, string content, Control showOn)
        {
            var action = new Action(() => ShowInternal(durationMs, isBalloon, icon, title, content, showOn));
            if (showOn.InvokeRequired) showOn.Invoke(action);
            else action();
        }

        public static void Show(int durationMs, bool isBalloon, ToolTipIcon icon, string title, string content, Control showOn, int x, int y)
        {
            var action = new Action(() => ShowInternal(durationMs, isBalloon, icon, title, content, showOn, new Point(x, y)));
            if (showOn.InvokeRequired) showOn.Invoke(action);
            else action();
        }
    }
}