﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace FormulaEnginePoC.DynamicExpressionParse
{
    internal class Signature : IEquatable<Signature>
    {
        public readonly DynamicProperty[] Properties;
        public readonly int HashCode;

        public Signature(IEnumerable<DynamicProperty> properties)
        {
            Properties = properties.ToArray();
            HashCode = 0;
            foreach (var p in Properties)
            {
                HashCode ^= p.Name.GetHashCode() ^ p.Type.GetHashCode();
            }
        }

        public override int GetHashCode()
        {
            return HashCode;
        }

        public override bool Equals(object obj)
        {
            return obj is Signature o && Equals(o);
        }

        public bool Equals(Signature other)
        {
            if (other == null) throw new ArgumentNullException(nameof(other));
            if (Properties.Length != other.Properties.Length) return false;
            for (var i = 0; i < Properties.Length; i++)
            {
                if (Properties[i].Name != other.Properties[i].Name ||
                    Properties[i].Type != other.Properties[i].Type) return false;
            }
            return true;
        }
    }
}