﻿using System.Collections.Generic;
using System.Linq;

namespace KaraokeRenderPoc.Karaoke
{
    internal class KaraokeLyric
    {
        private readonly List<Sentence> _sentences;
        public IReadOnlyList<Sentence> Sentences => _sentences;

        public int Length
        {
            get
            {
                var last = Sentences.Last().Words.Last();
                return last.StartPosition + last.Duration;
            }
        }

        public KaraokeLyric(IReadOnlyList<string> lines)
        {
            var sentences = new List<Sentence>();

            int position = 0;

            List<Word> words = null;
            int sentenceStartPosition = 0;

            foreach (var line in lines)
            {
                if (line == "=")
                {
                    if (words != null)
                    {
                        sentences.Add(new Sentence(sentenceStartPosition, words.ToArray()));
                    }
                    words = new List<Word>();
                    sentenceStartPosition = position;
                    continue;
                }

                var parts = line.Split('|');
                var duration = int.Parse(parts[2]);
                words.Add(new Word(parts[0], parts[1], position, duration));
                position += duration;
            }

            _sentences = sentences.ToList();
        }

        public int GetSentenceIndex(int position)
        {
            return _sentences.FindLastIndex(p => p.StartPosition <= position);
        }
    }
}